@@ -72,7 +72,7 @@ the following mandatory arguments:
 
 C<fh> the Perl I<filehandle> (not filedescriptor) to watch for
 events. C<poll> must be a string that is either C<r> or C<w>, that creates
-a watcher waiting for "r"eadable or "w"ritable events. C<cb> teh callback
+a watcher waiting for "r"eadable or "w"ritable events. C<cb> the callback
 to invoke everytime the filehandle becomes ready.
 
 Only one io watcher per C<fh> and C<poll> combination is allowed (i.e. on
@@ -111,7 +111,7 @@ Example:
    });
 
    # to cancel the timer:
-   undef $w
+   undef $w;
 
 =head2 CONDITION WATCHERS
 
@@ -254,7 +254,7 @@ use strict;
 
 use Carp;
 
-our $VERSION = '2.6';
+our $VERSION = '2.7';
 our $MODEL;
 
 our $AUTOLOAD;
@@ -307,7 +307,7 @@ sub detect() {
          }
 
          $MODEL
-           or die "No event module selected for AnyEvent and autodetect failed. Install any one of these modules: Event (or Coro+Event), Glib or Tk.";
+           or die "No event module selected for AnyEvent and autodetect failed. Install any one of these modules: EV (or Coro+EV), Event (or Coro+Event), Glib or Tk.";
       }
 
       unshift @ISA, $MODEL;
@@ -375,6 +375,7 @@ sub AnyEvent::Base::Signal::DESTROY {
 
 our %PID_CB;
 our $CHLD_W;
+our $CHLD_DELAY_W;
 our $PID_IDLE;
 our $WNOHANG;
 
@@ -387,6 +388,14 @@ sub _child_wait {
    undef $PID_IDLE;
 }
 
+sub _sigchld {
+   # make sure we deliver these changes "synchronous" with the event loop.
+   $CHLD_DELAY_W ||= AnyEvent->timer (after => 0, cb => sub {
+      undef $CHLD_DELAY_W;
+      &_child_wait;
+   });
+}
+
 sub child {
    my (undef, %arg) = @_;
 
@@ -400,9 +409,9 @@ sub child {
    }
 
    unless ($CHLD_W) {
-      $CHLD_W = AnyEvent->signal (signal => 'CHLD', cb => \&_child_wait);
-      # child could be a zombie already
-      $PID_IDLE ||= AnyEvent->timer (after => 0, cb => \&_child_wait);
+      $CHLD_W = AnyEvent->signal (signal => 'CHLD', cb => \&_sigchld);
+      # child could be a zombie already, so make at least one round
+      &_sigchld;
    }
 
    bless [$pid, $arg{cb}], "AnyEvent::Base::Child"
